/*
 * Copyright (c) 2010 Unimed Fortaleza. All Rights Reserved.
 * This software is the confidential and proprietary information of Unimed
 * Fortaleza ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Unimed Fortaleza.
 */
package br.com.unimedfortaleza.sabius.geradorrelatorio.componentes;

import java.util.Iterator;
import java.util.List;

import javax.el.ELContext;
import javax.el.ExpressionFactory;
import javax.el.MethodExpression;
import javax.el.ValueExpression;
import javax.faces.component.UIComponent;
import javax.faces.component.html.HtmlColumn;
import javax.faces.component.html.HtmlInputText;
import javax.faces.component.html.HtmlOutputLabel;
import javax.faces.component.html.HtmlOutputText;
import javax.faces.component.html.HtmlPanelGrid;
import javax.faces.context.FacesContext;

import org.ajax4jsf.component.html.HtmlActionParameter;
import org.ajax4jsf.component.html.HtmlAjaxCommandButton;
import org.ajax4jsf.component.html.HtmlAjaxRegion;
import org.ajax4jsf.component.html.HtmlAjaxSupport;
import org.richfaces.component.html.HtmlSuggestionBox;

import br.com.unimedfortaleza.sabius.common.util.SabiusUtil;
import br.com.unimedfortaleza.sabius.geradorrelatorio.domain.GeradorRelatorioParametro;
import br.com.unimedfortaleza.sabius.geradorrelatorio.domain.GeradorRelatorioParametroConfiguracao;
import br.com.unimedfortaleza.sabius.geradorrelatorio.domain.ObjetoComponenteDinamico;

/**
 * Classe responsvel pela criao do suggestion de Unimed.
 * 
 * @author Jorge Campos  BRQ Java Team
 * 
 *  History: 12-09-2011 - Jorge Campos
 * 
 */
public class GenericSuggestion {

	
	protected FacesContext context = FacesContext.getCurrentInstance();
	protected GeradorRelatorioParametro parametro;
	protected ObjetoComponenteDinamico objeto;
	
	public boolean isCodigoNumero;
	public boolean isDescricaoNumero;
	public Integer tamanhoCodigo = 5;
	public Integer maxCodigo = 0;
	public Integer tamanhoDescricao = 40;
	public Integer maxDescricao = 0;
	

	public GenericSuggestion(GeradorRelatorioParametro _parametro) {
		this.parametro = _parametro;
		objeto = new ObjetoComponenteDinamico();
		objeto.setSqlFindByKey(getSqlFindByKey(_parametro));
		objeto.setSqlAutoComplete(getSqlAutoComplete(_parametro));
		objeto.setSqlAutoCompleteLike(getSqlAutoCompleteLike(_parametro));
		objeto.setErro(getMsgErroString(_parametro));
		try{
			if ( !SabiusUtil.isNullOrEmpty(getConfig(_parametro, "codigonumero")) && 
					"true".equalsIgnoreCase(getConfig(_parametro, "codigonumero")) ){
				isCodigoNumero = true;
			}else{
				isCodigoNumero = false;
			}
			if ( !SabiusUtil.isNullOrEmpty(getConfig(_parametro, "descricaonumero")) && 
					"true".equalsIgnoreCase(getConfig(_parametro, "descricaonumero")) ){
				isDescricaoNumero = true;
			}else{
				isDescricaoNumero = false;
			}
			if ( !SabiusUtil.isNullOrEmpty(getConfig(_parametro, "tamanhocodigo")) ){
				tamanhoCodigo = Integer.parseInt(getConfig(_parametro, "tamanhocodigo"));
			}
			if ( !SabiusUtil.isNullOrEmpty(getConfig(_parametro, "maxcodigo")) ){
				maxCodigo = Integer.parseInt(getConfig(_parametro, "maxcodigo"));
			}
			if ( !SabiusUtil.isNullOrEmpty(getConfig(_parametro, "tamanhodescricao")) ){
				tamanhoDescricao = Integer.parseInt(getConfig(_parametro, "tamanhodescricao"));
			}
			if ( !SabiusUtil.isNullOrEmpty(getConfig(_parametro, "maxdescricao")) ){
				maxDescricao = Integer.parseInt(getConfig(_parametro, "maxdescricao"));
			}
		}catch( Exception e ){
			isCodigoNumero = false;
			isDescricaoNumero = false;
			tamanhoCodigo = 5;
			maxCodigo = 0; // 0 significa sem mximo
			tamanhoDescricao = 40;
			maxDescricao = 0; // 0 significa sem mximo
		}
	}
	
	public UIComponent buildComponent(){
		
		HtmlPanelGrid painel = new HtmlPanelGrid();
		painel.setColumns(3);
		painel.setColumnClasses("editCollum122pxRigth");
		painel.getChildren().add(buildCampoLabel());
		painel.getChildren().add(buildCampoCodigo());
		
		HtmlAjaxRegion ajaxRegion = new HtmlAjaxRegion();
		ajaxRegion.getChildren().add(buildCampoDescricao());
		ajaxRegion.getChildren().add(buildCampoSuggestion());
		
		ajaxRegion.getChildren().add(buildButtonCod());
		ajaxRegion.getChildren().add(buildButtonDesc());
		painel.getChildren().add(ajaxRegion);
		
		return painel;
	}
	
	private UIComponent buildCampoLabel(){
		HtmlOutputLabel label = new HtmlOutputLabel();
		label.setValue( parametro.getDescricao() + " :" );
		label.setRendererType("javax.faces.Text");
		return label;
	}
	
	private UIComponent buildCampoCodigo(){
		HtmlInputText campoCodigo = new HtmlInputText();
		campoCodigo.setId( parametro.getNome() );
		campoCodigo.setTitle( parametro.getHint() + " (Cdigo)" );
		campoCodigo.setRequired( parametro.getObrigatorio() );
		campoCodigo.setRequiredMessage( "O campo " + parametro.getDescricao() + " tem preenchimento obrigatrio" );

		campoCodigo.setValueExpression("value", context.getApplication().getExpressionFactory().  
		               createValueExpression(context.getELContext(), 
		            		   "#{geradorRelatorioManagerBean.paramComponentes['" + parametro.getNome() + "']}", String.class));
		
		if ( isCodigoNumero ){
			campoCodigo.setOnblur("soArrastaNumero(this);");
			campoCodigo.setOnkeypress("return(soEntraNumero(event,this));");
		}
		
		if (maxCodigo > 0)
			campoCodigo.setMaxlength( maxCodigo );
		
		campoCodigo.setSize( tamanhoCodigo );
		campoCodigo.setOnchange("jQuery('."+parametro.getNome()+"btnHidde').click();");
		/**
		 * Cdigo para adicionar o comportamento Ajax ao campo.
		 */
		HtmlAjaxSupport a4jSupportOnchange = new HtmlAjaxSupport();
		
		a4jSupportOnchange.setEvent("onchange");
		a4jSupportOnchange.setAjaxSingle( true );
		
		a4jSupportOnchange.setReRender( parametro.getNome() + ", desc" + parametro.getNome() + ", sugg" + parametro.getNome() + ", messagesRegion" );
		a4jSupportOnchange.setEventsQueue( parametro.getNome() + "Queue" );

		campoCodigo.getChildren().add(a4jSupportOnchange);
		
		return campoCodigo;
	}
	
	private UIComponent buildCampoDescricao(){
		HtmlInputText campoDescricao = new HtmlInputText();
		campoDescricao.setId("desc"+parametro.getNome());
		campoDescricao.setTitle( parametro.getHint() + " (Descrio)" );
		
		if ( isDescricaoNumero ){
			campoDescricao.setOnblur("soArrastaNumero(this);");
			campoDescricao.setOnkeypress("return(soEntraNumero(event,this));");
		}
		
		if ( maxDescricao > 0 )
			campoDescricao.setMaxlength( maxDescricao );
		
		campoDescricao.setSize( tamanhoDescricao );
		
		campoDescricao.setOnblur("jQuery('.desc"+parametro.getNome()+"btnHidde').click();");
		
		campoDescricao.setValueExpression("value", context.getApplication().getExpressionFactory().  
	               createValueExpression(context.getELContext(), 
	            		   "#{geradorRelatorioManagerBean.paramComponentes['desc" + parametro.getNome() + "']}", String.class));
		
		/**
		 * Definio da expressionFactory
		 */
		ExpressionFactory expressionFactory = context.getApplication().getExpressionFactory();
		ELContext elCtx = context.getELContext();
		
		/**
		 * Define um parmetro com o nome do suggestion para que ao usurio digitar no campo
		 * o bean do gerador saiba qual suggestion chamou o mtodo autoCompleteSuggestionLike
		 */
		HtmlActionParameter param = new HtmlActionParameter();
		param.setName("suggestionDesc");
		ValueExpression valorExpressParamSugg = expressionFactory
						.createValueExpression( elCtx, 
								parametro.getNome(), String.class);
		param.setValueExpression("value", valorExpressParamSugg);
		
		HtmlAjaxSupport a4jSupportOnchange = new HtmlAjaxSupport();
		a4jSupportOnchange.setEvent("oncomplete");
		a4jSupportOnchange.setReRender( parametro.getNome() + ", desc" + parametro.getNome() + ", sugg" + parametro.getNome() + ", messagesRegion" );
		a4jSupportOnchange.setAjaxSingle( true );
		a4jSupportOnchange.setOncomplete("jQuery('.desc"+parametro.getNome()+"btnHidde').click();");
		
		a4jSupportOnchange.getChildren().add(param);
		
		campoDescricao.getChildren().add( a4jSupportOnchange ); 
		
		return campoDescricao;
	}
	
	private UIComponent buildCampoSuggestion(){
		HtmlSuggestionBox campoSuggestion = new HtmlSuggestionBox();
		campoSuggestion.setId("sugg" + parametro.getNome());
		campoSuggestion.setVar( "_var" + parametro.getNome() );
		campoSuggestion.setWidth("350");
		campoSuggestion.setHeight("300");
		campoSuggestion.setFor( "desc" + parametro.getNome() );
		campoSuggestion.setNothingLabel("Nenhum registro encontrado.");
		campoSuggestion.setFrequency(0.1);
		campoSuggestion.setMinChars("2");
		campoSuggestion.setRequestDelay(800);
		
		/**
		 * Definio da expressionFactory
		 */
		ExpressionFactory expressionFactory = context.getApplication().getExpressionFactory();
		ELContext elCtx = context.getELContext();
		
		/**
		 * Define um parmetro com o nome do suggestion para que ao usurio digitar no campo
		 * o bean do gerador saiba qual suggestion chamou o mtodo autoCompleteSuggestionLike
		 */
		HtmlActionParameter param = new HtmlActionParameter();
		param.setName("suggestion");
		ValueExpression valorExpressParamSugg = expressionFactory
						.createValueExpression( elCtx, 
								parametro.getNome(), String.class);
		param.setValueExpression("value", valorExpressParamSugg);
		campoSuggestion.getChildren().add(param);
		
		/**
		 * Define o mtodo a ser chamado na digitao do edit (para preencher a lista do suggestion)
		 */
		MethodExpression me = expressionFactory.createMethodExpression(elCtx, 
					"#{geradorRelatorioManagerBean.autoCompleteSuggestionLike}", List.class, 
						new Class[]{Object.class});
		campoSuggestion.setSuggestionAction(me);
		
		/**
		 * Cdigo para adicionar o comportamento Ajax ao suggestion.
		 */
		HtmlAjaxSupport a4jSuggestion = new HtmlAjaxSupport();
		a4jSuggestion.setEvent("onselect");
		a4jSuggestion.setLimitToList(true); 
		a4jSuggestion.setAjaxSingle(true);
		a4jSuggestion.setRequestDelay(200);
		
		HtmlActionParameter param2 = new HtmlActionParameter();
		param2.setValue("jQuery(this).val();");
		param2.setName("suggACT"+parametro.getNome());
		ValueExpression valorExpressParam2Sugg = expressionFactory
						.createValueExpression( elCtx, 
								"#{geradorRelatorioManagerBean.paramComponentes['desc" + parametro.getNome() + "']}", String.class);
		param2.setAssignToBinding(valorExpressParam2Sugg);
		a4jSuggestion.getChildren().add(param2);
		
		campoSuggestion.getChildren().add(a4jSuggestion);
		
		/**
		 * Variveis para as colunas para o sugestion
		 */
		HtmlColumn columnDescricao = new HtmlColumn();
		columnDescricao.setId( campoSuggestion.getId() + "columnDescricao" );
		
		HtmlColumn columnCodigo = new HtmlColumn();
		columnCodigo.setId( campoSuggestion.getId() + "columnCodido" );

		/**
		 * Textos para as colunas no suggestion
		 */
		HtmlOutputText outDescricao = new HtmlOutputText();
		outDescricao.setId( columnDescricao.getId() + "outDesccricao" );
		
		HtmlOutputText outCodigo = new HtmlOutputText();
		outCodigo.setId( columnCodigo.getId() + "outCodigo" );
		
		/**
		 * Evaluate para as expresses, que mostra de fato os dados na listagem do suggestion
		 */
		ValueExpression setValorExpressDesc = expressionFactory
			.createValueExpression( elCtx, "#{" + campoSuggestion.getVar() + ".descricao}", String.class);
		outDescricao.setValueExpression("value", setValorExpressDesc);
		
		ValueExpression setValorExpressCod = expressionFactory
			.createValueExpression(elCtx, "#{" + campoSuggestion.getVar() + ".codigo}", String.class);
		outCodigo.setValueExpression("value", setValorExpressCod);

		/**
		 * Adiciona os textos nas colunas
		 */
		columnDescricao.getChildren().add( outDescricao );
		columnCodigo.getChildren().add( outCodigo );
		/**
		 * Adiciona as colunas no suggestion
		 */
		campoSuggestion.getChildren().add( columnDescricao );
		campoSuggestion.getChildren().add( columnCodigo );
		
		return campoSuggestion;
	}

	public ObjetoComponenteDinamico getObjeto() {
		return objeto;
	}
	
	public UIComponent buildButtonCod(){
		
		HtmlAjaxCommandButton buttonCod = new HtmlAjaxCommandButton();
		buttonCod.setValue("bt"+parametro.getNome());
		buttonCod.setStyle("display: none; position: absolute;");
		buttonCod.setStyleClass(parametro.getNome()+"btnHidde");
		buttonCod.setAjaxSingle(true);
		buttonCod.setLimitToList(true);

		/**
		 * Cdigo para adicionar o comportamento Ajax ao campo.
		 */
		HtmlAjaxSupport a4jSupport = new HtmlAjaxSupport();
		
		ExpressionFactory factory = context.getApplication().getExpressionFactory();
		MethodExpression methodExpression = factory.createMethodExpression(context.getELContext(), 
						"#{geradorRelatorioManagerBean.findByKey( '" + parametro.getNome() + "', 'COD' )}", null, new Class<?>[0]);
		a4jSupport.setActionExpression(methodExpression);
		a4jSupport.setEvent("oncomplete");
		a4jSupport.setImmediate(true); 
		a4jSupport.setAjaxSingle( true );
		a4jSupport.setReRender( parametro.getNome() + ", desc" + parametro.getNome() + ", sugg" + parametro.getNome() + ", messagesRegion" );
		a4jSupport.setEventsQueue( parametro.getNome() + "queueBt" );
		
		buttonCod.getChildren().add(a4jSupport);
		
		return buttonCod;
		
	}
	
	public UIComponent buildButtonDesc(){
		
		HtmlAjaxCommandButton buttonDesc = new HtmlAjaxCommandButton();
		
		buttonDesc.setValue("btdesc"+parametro.getNome());
		buttonDesc.setStyle("display: none; position: absolute;");
		buttonDesc.setStyleClass("desc"+parametro.getNome()+"btnHidde");

		buttonDesc.setAjaxSingle(true);
		buttonDesc.setLimitToList(true);

		/**
		 * Cdigo para adicionar o comportamento Ajax ao campo.
		 */
		HtmlAjaxSupport a4jCodUnimed = new HtmlAjaxSupport();
		
		ExpressionFactory factory = context.getApplication().getExpressionFactory();
		MethodExpression methodExpression = factory.createMethodExpression(context.getELContext(), 
						"#{geradorRelatorioManagerBean.findByKey( '" + parametro.getNome() + "', 'DESC' )}", null, new Class<?>[0]);
		a4jCodUnimed.setActionExpression(methodExpression);
		a4jCodUnimed.setEvent("oncomplete");
		a4jCodUnimed.setImmediate(true); 
		a4jCodUnimed.setAjaxSingle( true );
		a4jCodUnimed.setReRender( parametro.getNome() + ", desc" + parametro.getNome() + ", sugg" + parametro.getNome() + ", messagesRegion" );
		a4jCodUnimed.setEventsQueue( "desc"+parametro.getNome() + "queueBt" );
		
		buttonDesc.getChildren().add(a4jCodUnimed);
		
		return buttonDesc;
		
	}
	
	/**
	 * Este mtodo verifica se o parmetro possui uma configurao de consulta findbykey cadastrada para o parmetro
	 * e retorna o campo SQL para a configurao solicitada
	 * @param parametro GeradorRelatorioParametro
	 * @return campo SQL da configurao
	 */
	protected String getSqlFindByKey(GeradorRelatorioParametro parametro){
		String retorno = "";
		if ( !SabiusUtil.isNull(parametro)
				&& !SabiusUtil.isNull(parametro.getParametrosConfiguracoes())
				&& parametro.getParametrosConfiguracoes().size()>0 ){
			Iterator<GeradorRelatorioParametroConfiguracao> itCfg = parametro.getParametrosConfiguracoes().iterator();
			while ( itCfg.hasNext() ){
				GeradorRelatorioParametroConfiguracao cfg = itCfg.next();
				if ( "findbykey".equalsIgnoreCase(cfg.getAtributo()) ){
					retorno = cfg.getSql();
					break;
				}
			}
		}
		return retorno;
	}
	
	/**
	 * Este mtodo verifica se o parmetro possui uma configurao de consulta autocomplete cadastrada para o parmetro
	 * e retorna o campo SQL para a configurao solicitada
	 * @param parametro GeradorRelatorioParametro
	 * @return campo SQL da configurao
	 */
	protected String getSqlAutoComplete(GeradorRelatorioParametro parametro){
		String retorno = "";
		if ( !SabiusUtil.isNull(parametro)
				&& !SabiusUtil.isNull(parametro.getParametrosConfiguracoes())
				&& parametro.getParametrosConfiguracoes().size()>0 ){
			Iterator<GeradorRelatorioParametroConfiguracao> itCfg = parametro.getParametrosConfiguracoes().iterator();
			while ( itCfg.hasNext() ){
				GeradorRelatorioParametroConfiguracao cfg = itCfg.next();
				if ( "autocomplete".equalsIgnoreCase(cfg.getAtributo()) ){
					retorno = cfg.getSql();
					break;
				}
			}
		}
		return retorno;
	}

	/**
	 * Este mtodo verifica se o parmetro possui uma configurao de consulta autocompletelike cadastrada para o parmetro
	 * e retorna o campo SQL para a configurao solicitada
	 * @param parametro GeradorRelatorioParametro
	 * @return campo SQL da configurao
	 */
	protected String getSqlAutoCompleteLike(GeradorRelatorioParametro parametro){
		String retorno = "";
		if ( !SabiusUtil.isNull(parametro)
				&& !SabiusUtil.isNull(parametro.getParametrosConfiguracoes())
				&& parametro.getParametrosConfiguracoes().size()>0 ){
			Iterator<GeradorRelatorioParametroConfiguracao> itCfg = parametro.getParametrosConfiguracoes().iterator();
			while ( itCfg.hasNext() ){
				GeradorRelatorioParametroConfiguracao cfg = itCfg.next();
				if ( "autocompletelike".equalsIgnoreCase(cfg.getAtributo()) ){
					retorno = cfg.getSql();
					break;
				}
			}
		}
		return retorno;
	}	
	
	/**
	 * Este mtodo verifica se o parmetro possui uma configurao de mensagem de erro cadastrada para o parmetro
	 * e retorna o campo Valor para a configurao solicitada
	 * @param parametro GeradorRelatorioParametro
	 * @return campo SQL da configurao
	 */
	protected String getMsgErroString(GeradorRelatorioParametro parametro){
		String retorno = "";
		if ( !SabiusUtil.isNull(parametro)
				&& !SabiusUtil.isNull(parametro.getParametrosConfiguracoes())
				&& parametro.getParametrosConfiguracoes().size()>0 ){
			Iterator<GeradorRelatorioParametroConfiguracao> itCfg = parametro.getParametrosConfiguracoes().iterator();
			while ( itCfg.hasNext() ){
				GeradorRelatorioParametroConfiguracao cfg = itCfg.next();
				if ( "msgerro".equalsIgnoreCase(cfg.getAtributo()) ){
					retorno = cfg.getValor();
					break;
				}
			}
		}
		return retorno;
	}
	
	/**
	 * Este mtodo verifica se o parmetro possui uma configurao cadastrada para o parmetro
	 * e retorna o campo Valor para a configurao solicitada
	 * @param parametro GeradorRelatorioParametro
	 * @return campo SQL da configurao
	 */
	protected String getConfig(GeradorRelatorioParametro parametro, String config){
		String retorno = "";
		if ( !SabiusUtil.isNull(parametro)
				&& !SabiusUtil.isNull(parametro.getParametrosConfiguracoes())
				&& parametro.getParametrosConfiguracoes().size()>0 ){
			Iterator<GeradorRelatorioParametroConfiguracao> itCfg = parametro.getParametrosConfiguracoes().iterator();
			while ( itCfg.hasNext() ){
				GeradorRelatorioParametroConfiguracao cfg = itCfg.next();
				if ( config.equalsIgnoreCase(cfg.getAtributo()) ){
					retorno = cfg.getValor();
					break;
				}
			}
		}
		return retorno;
	}
	
}
